@@ -24,6 +24,7 @@ namespace GLib {
 
 	using System;
 	using System.Collections;
+	using System.IO;
 	using System.Reflection;
 	using System.Runtime.InteropServices;
 
@@ -37,6 +38,11 @@ namespace GLib {
 			this.val = val;
 		}
 
+		public static GType FromName (string native_name)
+		{
+			return new GType (g_type_from_name (native_name));
+		}
+		
 		public static readonly GType Invalid = new GType ((IntPtr) TypeFundamentals.TypeInvalid);
 		public static readonly GType None = new GType ((IntPtr) TypeFundamentals.TypeNone);
 		public static readonly GType Interface = new GType ((IntPtr) TypeFundamentals.TypeInterface);
@@ -45,6 +51,8 @@ namespace GLib {
 		public static readonly GType Boolean = new GType ((IntPtr) TypeFundamentals.TypeBoolean);
 		public static readonly GType Int = new GType ((IntPtr) TypeFundamentals.TypeInt);
 		public static readonly GType UInt = new GType ((IntPtr) TypeFundamentals.TypeUInt);
+		public static readonly GType Long = new GType ((IntPtr) TypeFundamentals.TypeLong);
+		public static readonly GType ULong = new GType ((IntPtr) TypeFundamentals.TypeULong);
 		public static readonly GType Int64 = new GType ((IntPtr) TypeFundamentals.TypeInt64);
 		public static readonly GType UInt64 = new GType ((IntPtr) TypeFundamentals.TypeUInt64);
 		public static readonly GType Enum = new GType ((IntPtr) TypeFundamentals.TypeEnum);
@@ -54,6 +62,7 @@ namespace GLib {
 		public static readonly GType String = new GType ((IntPtr) TypeFundamentals.TypeString);
 		public static readonly GType Pointer = new GType ((IntPtr) TypeFundamentals.TypePointer);
 		public static readonly GType Boxed = new GType ((IntPtr) TypeFundamentals.TypeBoxed);
+		public static readonly GType Param = new GType ((IntPtr) TypeFundamentals.TypeParam);
 		public static readonly GType Object = new GType ((IntPtr) TypeFundamentals.TypeObject);
 
 		static Hashtable types = new Hashtable ();
@@ -143,6 +152,32 @@ namespace GLib {
 			// cctor already calls g_type_init.
 		}
 
+		static Type FindTypeInReferences (string type_name, Assembly asm, Hashtable visited)
+		{
+			if (visited.Contains (asm))
+				return null;
+
+			visited [asm] = asm;
+			Type result = asm.GetType (type_name);
+			if (result == null) {
+				if (asm is System.Reflection.Emit.AssemblyBuilder)
+					/* These don't support Location */
+					return null;
+				string asm_dir = Path.GetDirectoryName (asm.Location);
+				foreach (AssemblyName ref_name in asm.GetReferencedAssemblies ()) {
+					Assembly ref_asm;
+					if (File.Exists (Path.Combine (asm_dir, ref_name.Name + ".dll")))
+						ref_asm = Assembly.LoadFrom (Path.Combine (asm_dir, ref_name.Name + ".dll"));
+					else
+						ref_asm = Assembly.Load (ref_name);
+					result = FindTypeInReferences (type_name, ref_asm, visited);
+					if (result != null)
+						break;
+				}
+			}
+			return result;
+		}
+
 		public static Type LookupType (IntPtr typeid)
 		{
 			if (types.Contains (typeid))
@@ -151,12 +186,22 @@ namespace GLib {
 			string native_name = Marshaller.Utf8PtrToString (g_type_name (typeid));
 			string type_name = GetQualifiedName (native_name);
 			Type result = null;
-			foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies ()) {
+			Assembly[] assemblies = (Assembly[]) AppDomain.CurrentDomain.GetAssemblies ().Clone ();
+			foreach (Assembly asm in assemblies) {
 				result = asm.GetType (type_name);
 				if (result != null)
 					break;
 			}
 
+			if (result == null) {
+				Hashtable visited = new Hashtable ();
+				foreach (Assembly asm in assemblies) {
+					result = FindTypeInReferences (type_name, asm, visited);
+					if (result != null)
+						break;
+				}
+			}
+
 			Register (new GType (typeid), result);
 			return result;
 		}
@@ -192,6 +237,9 @@ namespace GLib {
 
 		[DllImport("libgobject-2.0-0.dll")]
 		static extern IntPtr g_type_name (IntPtr raw);
+		
+		[DllImport("libgobject-2.0-0.dll")]
+		static extern IntPtr g_type_from_name (string name);
 
 		public override string ToString ()
 		{
