@@ -23,14 +23,14 @@ namespace Pango {
 
 	public class AttrStretch : Attribute {
 
-		[DllImport("libpango-1.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libpango-1.0-0.dll")]
 		static extern IntPtr pango_attr_stretch_new (Pango.Stretch stretch);
 
 		public AttrStretch (Pango.Stretch stretch) : this (pango_attr_stretch_new (stretch)) {}
 
 		internal AttrStretch (IntPtr raw) : base (raw) {}
 
-		[DllImport("pangosharpglue-2", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("pangosharpglue-2")]
 		static extern int pangosharp_attr_int_get_value (IntPtr raw);
 
 		public Pango.Stretch Stretch {
